///////////////////////////////////////////////////////////////////////////////
//
// Procedury wspierajace wykrywanie debuggera oraz monitorow systemowych
//
// Bartosz Wojcik | http://www.pelock.com
//
///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdio.h>

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// WYKRYWANIE DEBUGGERA POPRZEZ STEROWNIKI
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// HANDLE CheckDriver(char *lpszDriver)
//
// funkcja probuje otworzyc dostep do podanego sterownika systemowego
//
// na wyjsciu:
// uchwyt sterownika lub INVALID_HANDLE_VALUE w przypadku braku sterownika
//
///////////////////////////////////////////////////////////////////////////////

HANDLE CheckDriver(char *lpszDriver)
{
	return ( CreateFile(lpszDriver, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL) );
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsSice()
//
// funkcja sprawdza obecnosc debuggera SoftIce w wersji dla systemow
// Windows 9x
//
// na wyjsciu:
// jesli debugger SoftIce jest aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsSice()
{
	return ( CheckDriver("\\\\.\\SICE") != INVALID_HANDLE_VALUE ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsNtice()
//
// funkcja sprawdza obecnosc debuggera SoftIce w wersji dla systemow
// Windows NT/2K/XP
//
// na wyjsciu:
// jesli debugger SoftIce jest aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsNtice()
{
	return ( CheckDriver("\\\\.\\NTICE") != INVALID_HANDLE_VALUE ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsSiwdebug()
//
// funkcja sprawdza obecnosc dodatkowego sterownika zainstalowanego
// wraz z debuggerem SoftIce dla systemow Windows 9x, obecnosc tego
// sterownika w systemie, swiadczy o obecnosci debuggera
//
// na wyjsciu:
// jesli debugger SoftIce jest aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsSiwdebug()
{
	// otworz pomocniczy sterownik
	CheckDriver("\\\\.\\SIWDEBUG");

	return ( GetLastError() != ERROR_FILE_NOT_FOUND ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// WYKRYWANIE DEBUGGERA PRZEZ WPISY W REJESTRZE WINDOWS
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsReg()
//
// funkcja sprawdza obecnosc galezi rejestru Windows
//
// na wyjsciu:
// jesli w rejestrze Windows znajduje sie podana galaz, funkcja zwroci
// TRUE w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsReg(char *lpszReg)
{
	HKEY phkResult;
	DWORD iResult;

	// otworz galaz rejestru Windows w HKEY_LOCAL_MACHINE
	iResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, lpszReg, 0, KEY_ALL_ACCESS, &phkResult);

	// zamknij uchwyt
	RegCloseKey(phkResult);

	return (iResult == ERROR_SUCCESS ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsSiceReg1()
//
// funkcja sprawdza obecnosc galezi rejestru wykorzystywanych przez
// debugger SoftIce do trzymania informacji potrzebnych przy
// deinstalacji debuggera
//
// na wyjsciu:
// jesli w rejestrze Windows znajduje sie galaz debuggera, funkcja zwroci
// TRUE w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsSiceReg1()
{
	return IsReg("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\SoftICE");
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsSiceReg2()
//
// funkcja sprawdza obecnosc galezi rejestru wykorzystywanych przez
// debugger SoftIce do trzymania informacji konfiguracyjnych
//
// na wyjsciu:
// jesli w rejestrze Windows znajduje sie galaz debuggera, funkcja zwroci
// TRUE w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsSiceReg2()
{
	return IsReg("SOFTWARE\\NuMega\\SoftICE");
}


///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsBpx(char *lpszModule,char *lpszFunction)
//
// funkcja sprawdza, czy na adresie podanej funkcji zastawiona
// pulapka debuggera
//
// na wyjsciu:
// jesli pulapka jest zastawiona, funkcja zwraca TRUE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsBpx(char *lpszModule,char *lpszFunction)
{
	HINSTANCE hLibrary;
	DWORD lpProc;

	// sprawdz czy biblioteka jest juz zaladowana
	hLibrary = GetModuleHandle(lpszModule);

	// zaladuj biblioteke
	if (hLibrary == NULL)
	{
		hLibrary = LoadLibrary(lpszModule);
	}

	if (hLibrary != NULL)
	{
		// pobierz adres procedury
		lpProc = (DWORD)GetProcAddress(hLibrary, lpszFunction);

		if (lpProc != 0)
		{
			// sprawdz pierwszy bajt procedury na
			// obecnosc pulapki
			if (*(BYTE *)lpProc == 0xCC) return TRUE;
		}

	}

	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsAutoexec(char *lpString)
//
// funkcja szuka w pliku autoexec.bat podanych wpisow
//
// na wyjsciu:
// jesli znaleziono podany wpis, funkcja zwroci TRUE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsAutoexec(char *lpString)
{
	BOOL bResult = FALSE;
	int iSize, i;
	char *lpMap, *lpAutoexec, lpFile[256];
	HANDLE hFile, hMap;

	// odczytaj sciezke katalogu Windows
	GetWindowsDirectory(lpFile, 256);

	// zbuduj sciezke do pliku autoexec.bat
	strcpy(&lpFile[3], "autoexec.bat");


	// otworz plik
	hFile = CreateFile(lpFile, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if (hFile != INVALID_HANDLE_VALUE)
	{
		// pobierz rozmiar pliku
		iSize = GetFileSize(hFile, NULL);

		// jesli rozmiar 0 wyjdz
		if (iSize)
		{
			// utworz mape pliku
			hMap = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);

			if (hMap)
			{
				// mapuj plik do pamieci
				lpMap = MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0);

				if (lpMap != 0)
				{
					lpAutoexec = lpMap;

					// skanuj zawartosc pliku autoexec.bat
					while (iSize--)
					{
						if (strnicmp(lpAutoexec++, lpString, strlen(lpString)) == 0)
						{
							bResult = TRUE;
							break;
						}
					}
				}

				UnmapViewOfFile(lpMap);
			}

			CloseHandle(hMap);
		}

		CloseHandle(hFile);
	}

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// WYKRYWANIE MONITOROW
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsFilemon()
//
// funkcja sprawdza obecnosc sterownika wykorzystanego w programie
// sluzacym do sledzenia odwolan do plikow (filemon)
//
// na wyjsciu:
// jesli monitor plikow jest aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsFilemon()
{
	return ( CheckDriver("\\\\.\\FILEVXD") != INVALID_HANDLE_VALUE ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsRegmon()
//
// funkcja sprawdza obecnosc sterownika wykorzystanego w programie
// sluzacym do sledzenia odwolan do rejestru windows (regmon)
//
// na wyjsciu:
// jesli monitor rejestru jest aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsRegmon()
{
	return ( CheckDriver("\\\\.\\REGVXD") != INVALID_HANDLE_VALUE ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsFilemonWnd()
//
// funkcja sprawdza, czy obecnie otwarte jest okno Filemon'a
//
// na wyjsciu:
// jesli monitor plikow jest aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsFilemonWnd()
{
	return ( FindWindowEx(NULL, NULL, NULL, "File Monitor - Sysinternals: www.sysinternals.com") != NULL ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsRegmonWnd()
//
// funkcja sprawdza, czy obecnie otwarte jest okno Regmon'a
//
// na wyjsciu:
// jesli monitor rejestru jest aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsRegmonWnd()
{
	return ( FindWindowEx(NULL, NULL, NULL, "Registry Monitor - Sysinternals: www.sysinternals.com") != NULL ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL IsOllyDbgWnd()
//
// funkcja sprawdza, czy obecnie otwarte jest okno debuggera OllyDbg
//
// na wyjsciu:
// jesli debugger OllyDbg aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL IsOllyDbgWnd()
{
	return ( FindWindowEx(NULL, NULL, NULL, "OllyDbg") != NULL ? TRUE : FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL KillFilemonWnd()
//
// funkcja w razie wykrycia okna Filemona zamyka je
//
// na wyjsciu:
// jesli monitor rejestru byl aktywny i zostal zamkniety, funkcja
// zwraca TRUE, w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL KillFilemonWnd()
{
	HANDLE hWindow, hProcess;
	DWORD dwPid;

	// znajdz okno Filemona
	hWindow = FindWindowEx(NULL, NULL, NULL, "Registry Monitor - Sysinternals: www.sysinternals.com");

	// w razie znalezienia okna
	if (hWindow != NULL)
	{
		// pobierz indentyfikator procesu, do ktorego nalezy okno
		if (!GetWindowThreadProcessId(hWindow, &dwPid))
		{
			// otworz proces z flagami pozwalajacymi wylaczyc go
			hProcess = OpenProcess(PROCESS_TERMINATE, FALSE, dwPid);

			if (hProcess != NULL)
			{
				// zakoncz proces
				if ( TerminateProcess(hProcess,-1) != 0) return TRUE;
			}
		}
	}

	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
// BOOL KillRegmonWnd()
//
// funkcja w razie wykrycia okna Regmona zamyka je
//
// na wyjsciu:
// jesli monitor rejestru byl aktywny, funkcja zwraca TRUE,
// w przeciwnym wypadku FALSE
//
///////////////////////////////////////////////////////////////////////////////

BOOL KillRegmonWnd()
{
	HANDLE hWindow;

	// znajdz okno Regmona
	hWindow = FindWindowEx(NULL, NULL, NULL, "Registry Monitor - Sysinternals: www.sysinternals.com");

	// w razie znalezienia okna
	if (hWindow != NULL)
	{
		SendMessage(hWindow, WM_CLOSE, 0, 0);
	}

	return ( hWindow != 0? TRUE : FALSE);
}

void ShowMsg(char *lpszMessage)
{
	MessageBox(NULL, lpszMessage, "Uwaga!", MB_ICONEXCLAMATION);
}

void main()
{

	// wykrywanie debuggera przez sprawdzenie sterownika
	if (IsSice())        ShowMsg("IsSice(): Wykryto debuggera SoftIce!");
	if (IsSiwdebug())    ShowMsg("IsSiwdebug(): Wykryto pomocniczy sterownik debuggera SoftIce!");
	if (IsNtice())       ShowMsg("IsNtice(): Wykryto debuggera SoftIce NT!");

	// wykrywanie debuggera przez wpisy w rejestrze
	if (IsSiceReg1())    ShowMsg("IsSiceReg1(): Wykryto wpis w rejestrze debuggera SoftIce!");
	if (IsSiceReg2())    ShowMsg("IsSiceReg2(): Wykryto wpis w rejestrze debuggera SoftIce!");

	// wykrywanie monitorow poprzez sterowniki
	if (IsFilemon())     ShowMsg("IsFilemon(): Wykryto monitor Filemon!!");
	if (IsRegmon())      ShowMsg("IsRegmon(): Wykryto monitor Regmon!");

	// wykrywanie monitorow poprzez nazwy glownych okien
	if (IsFilemonWnd())  ShowMsg("IsFilemonWnd(): Wykryto monitor Filemon!!");
	if (IsRegmonWnd())   ShowMsg("IsRegmonWnd(): Wykryto monitor Regmon!");

	// wykrywanie okna debuggera OllyDbg
	if (IsOllyDbgWnd())   ShowMsg("IsOllyDbgWnd(): Wykryto debugger OllyDbg!");

	// zamykanie okna Filemona przez TerminateProcess
	if (KillFilemonWnd()) ShowMsg("KillFilemonWnd(): Proces monitora Filemon zostal zakonczony!");

	// zamykanie okna Regmona przez WM_CLOSE
	if (KillRegmonWnd()) ShowMsg("KillRegmonWnd(): Okno Regmona zostalo zamkniete!");

	// sprawdzanie obecnosci pulapki debuggera
	if (IsBpx("USER32.dll","GetDlgItemTextA")) ShowMsg("Wykryto aktywna pulapke zaloznona na funkcje GetDlgItemText!");

	// wykrywanie debuggera SoftIce NT
	if (IsBpx("KERNEL32.dll","SetUnhandledExceptionFilter")) ShowMsg("Wykryto aktywnego debuggera SoftIce NT!");
	
	// wykrywanie wpisow debuggera w autoexec.bat
	if (IsAutoexec("nume") || IsAutoexec("softic")) ShowMsg("Wykryto wpis debuggera w autoexec.bat!");
}
